import * as plugins from './test-sdk.plugins';
import { AgTestServer } from './test-sdk.classes.testserver';

export {
  AgTestServer
};

let testServer: AgTestServer;
export const createTestServer = async (handlerArg: plugins.agSdk.AAgHandler<any>) => {
  testServer = new AgTestServer(handlerArg);
  await testServer.start();
  return testServer;
};

export const testFire = <
  A extends plugins.agSdk.AAgHandler<any>,
  T extends plugins.typedrequestInterfaces.ITypedRequest
>(
  slugArg: A['slug'],
  methodArg: T['method'],
  requestArg: T['request']
) => {
  if (!testServer) {
    throw new Error('you need to create and start a testServer first!');
  }
  if (testServer.server.serverStatus !== 'running') {
    throw new Error('you need to start the testServer first!');
  }
  const typedRequest = new plugins.typedrequest.TypedRequest<T>(
    `http://localhost:${testServer.server.options.port}/${slugArg}`,
    methodArg
  );
  const responsePromise = typedRequest.fire(requestArg);
  const expect = async (expectedResponseArg: T['response']) => {
    const actualResponse = await responsePromise;
    const comparisonResult = plugins.smartobject.compareObjects(expectedResponseArg, actualResponse);
    let throwErrorBool = false;
    if (comparisonResult.divergingProperties.length > 0) {
      console.log(`The following properties diverged:`);
      console.log(comparisonResult.divergingProperties);
      for (const divProperty of comparisonResult.divergingProperties) {
        if (divProperty !== 'maydiverge') {
          throwErrorBool = true;
        } else {
          console.log(`${divProperty} may diverge, not throwing for this one`);
        }
      }
    }
    if (comparisonResult.missingProperties.length > 0) {
      console.log(`The following properties diverged:`);
      console.log(comparisonResult.divergingProperties);
      throwErrorBool = true;
    }
    if (throwErrorBool) {
      throw new Error('response did not comply');
    }
  };
  return {
    expect
  };
};

export const stopTestServer = async () => {
  if (testServer) {
    await testServer.stop();
  }
};
